@@ -1,5 +1,8 @@
 package Test::Run::Straps::StrapsTotalsObj;
 
+use strict;
+use warnings;
+
 =head1 NAME
 
 Test::Run::Straps::StrapsTotalsObj - an object representing the totals of the
@@ -9,13 +12,11 @@ straps class.
 
 =cut
 
-use vars qw(@fields);
+use Test::Run::Straps::StrapsDetailsObj;
 
 use base 'Test::Run::Base::Struct';
 
-use Test::Run::Assert;
-
-use Test::Run::Straps::StrapsDetailsObj;
+use vars qw(@fields);
 
 @fields = (qw(
     bonus
@@ -35,13 +36,360 @@ use Test::Run::Straps::StrapsDetailsObj;
     wait
 ));
 
-sub _get_fields
+sub _get_private_fields
 {
     return [@fields];
 }
 
 __PACKAGE__->mk_accessors(@fields);
 
+=head1 METHODS
+
+=head2 $self->_calc_passing()
+
+Calculates whether the test file has passed.
+
+=cut
+
+sub _is_skip_all
+{
+    my $self = shift;
+    
+    return (($self->max() == 0) && defined($self->skip_all()));
+}
+
+sub _is_all_tests_passed
+{
+    my $self = shift;
+
+    return
+    (
+        $self->max && $self->seen
+        && ($self->max == $self->seen)
+        && ($self->max == $self->ok)
+    );
+}
+
+sub _calc_passing
+{
+    my $self = shift;
+
+    return ($self->_is_skip_all() || $self->_is_all_tests_passed());
+}
+
+=head2 $self->determine_passing()
+
+Calculates whether the test file has passed and caches it in the passing()
+slot.
+
+=cut
+
+sub determine_passing
+{
+    my $self = shift;
+    $self->passing($self->_calc_passing() ? 1 : 0);
+}
+
+=head2 $self->last_detail()
+
+Returns the last detail.
+
+=cut
+
+sub last_detail
+{
+    my $self = shift;
+
+    return $self->details->[-1];
+}
+
+sub _calc_enormous_event_num
+{
+    my $self = shift;
+
+    return 100_000;
+}
+
+sub _is_enormous_event_num
+{
+    my $self = shift;
+
+    my $large_num = $self->_calc_enormous_event_num();
+
+    return
+        +($self->_event->number > $large_num)
+            &&
+         ($self->_event->number > ($self->max || $large_num))
+        ;
+}
+
+sub _init_details_obj_instance
+{
+    my ($self, $args) = @_;
+    return Test::Run::Straps::StrapsDetailsObj->new($args);
+}
+
+sub _handle_event_main
+{
+    my $self = shift;
+
+    $self->_inc_seen();
+    $self->_update_by_labeled_test_event();
+    $self->_update_if_pass();
+    $self->_update_details_wrapper();
+}
+
+sub _def_or_blank
+{
+    my $value = shift;
+
+    return defined($value) ? $value : "";
+}
+
+sub _defined_hash_values
+{
+    my ($self, $hash) = @_;
+
+    return
+    {
+        map 
+        { $_ => _def_or_blank($hash->{$_}) }
+        keys(%$hash)
+    };
+}
+
+sub _calc_always_def_details_initializer
+{
+    my $self = shift;
+
+    my $event = $self->_event;
+
+    return 
+    {
+        actual_ok => scalar($event->is_ok()),
+        name => $event->description,
+        type => lc($event->directive),
+        reason => $event->explanation,
+    };
+}
+
+sub _calc_defined_details
+{
+    my $self = shift;
+
+    $self->_defined_hash_values(
+        $self->_calc_always_def_details_initializer()
+    );
+}
+
+sub _calc_details
+{
+    my $self = shift;
+
+    my $event = $self->_event;
+
+    return
+        $self->_init_details_obj_instance(
+            {
+                ok => $self->_is_event_pass(),
+                %{$self->_calc_defined_details()},
+            }
+        );
+}
+
+sub _update_details
+{
+    my ($self) = @_;
+
+    $self->details->[$self->_event->number - 1] = $self->_calc_details();
+
+    return ;
+}
+
+sub _update_skip_event
+{
+    my $self = shift;
+
+    $self->inc_field('skip');
+
+    return;
+}
+
+sub _is_event_todo
+{
+    my $self = shift;
+
+    return $self->_event->has_todo();
+}
+
+sub _update_if_pass
+{
+    my $self = shift;
+
+    if ($self->_is_event_pass())
+    {
+        $self->inc_field('ok');
+    }
+}
+
+sub _is_event_pass
+{
+    my $self = shift;
+
+    return
+    (
+        $self->_event->is_ok() || 
+        $self->_is_event_todo() || 
+        $self->_event->has_skip()
+    );
+}
+
+sub _handle_enormous_event_num
+{
+    my $self = shift;
+
+    return $self->_enormous_num_cb->();
+}
+
+sub _update_todo_event
+{
+    my $self = shift;
+
+    $self->inc_field('todo');
+
+    if ($self->_event->is_actual_ok())
+    {
+        $self->inc_field('bonus');
+    }
+
+    return;
+}
+
+
+sub _inc_seen
+{
+    my $self = shift;
+
+    $self->inc_field('seen');
+}
+
+=head2 $self->_handle_event({event => $event, enormous_num_cb => sub {...}});
+
+Updates the state of the details using a new TAP::Parser event - $event .
+C<enormous_num_cb> points to a subroutine reference that is the callback for
+handling enormous numbers.
+
+=cut
+
+sub _setup_event
+{
+	my ($self, $args) = @_;
+
+	$self->_event($args->{event});
+    $self->_enormous_num_cb($args->{enormous_num_cb});
+
+	return ;
+}
+
+sub _detach_event
+{
+    my ($self) = @_;
+
+	$self->_event(undef);
+    $self->_enormous_num_cb(undef);
+}
+
+sub handle_event
+{
+    my ($self, $args) = @_;
+
+    $self->_setup_event($args);
+
+    $self->_handle_event_main();
+
+    $self->_detach_event();
+}
+
+sub _update_details_wrapper
+{
+    my $self = shift;
+
+    if ($self->_is_enormous_event_num())
+    {
+        $self->_handle_enormous_event_num();
+    }
+    else
+    {
+        $self->_update_details();
+    }
+}
+
+sub _update_by_labeled_test_event
+{
+    my $self = shift;
+
+    if ($self->_event->has_todo())
+    {
+        $self->_update_todo_event();
+    }
+    elsif ($self->_event->has_skip())
+    {
+        $self->_update_skip_event();
+    }
+
+    return;
+}
+
+=head2 $self->update_skip_reason($detail)
+
+Updates the skip reason according to the detail $detail.
+
+=cut
+
+sub _get_skip_reason
+{
+    my ($self, $detail) = @_;
+
+    if (!defined($self->skip_reason))
+    {
+        return $detail->reason();
+    }
+    elsif ($self->skip_reason ne $detail->reason())
+    {
+        return "various reasons";
+    }
+    else
+    {
+        return $self->skip_reason;
+    }
+}
+
+sub _real_update_skip_reason
+{
+    my ($self, $detail) = @_;
+
+    $self->skip_reason($self->_get_skip_reason($detail));
+}
+
+sub update_skip_reason
+{
+    my ($self, $detail) = @_;
+
+    if ($detail->type eq "skip")
+    {
+        $self->_real_update_skip_reason($detail);
+    }
+}
+
+sub _get_failed_details
+{
+    my $self = shift;
+
+    my $details = $self->details;
+
+    return [ grep {! $details->[$_-1]->{ok} } (1 .. @$details) ];
+}
+
 =head2 $self->bonus()
 
 Number of TODO tests that unexpectedly passed.
@@ -96,275 +444,15 @@ The number of "Todo" tests that were encountered.
 
 The wait code of the test script.
 
-=cut
-
-sub _def_or_blank {
-    return $_[0] if defined $_[0];
-    return "";
-}
-
-=head1 METHODS
-
-=head2 $self->_calc_passing()
-
-Calculates whether the test file has passed.
-
-=cut
-
-sub _calc_passing
-{
-    my $self = shift;
-    return 
-    (
-        ($self->max() == 0 &&  defined $self->skip_all()) ||
-        (
-            $self->max() && $self->seen() &&
-            $self->max() == $self->seen() &&
-            $self->max() == $self->ok()
-        )
-    );
-}
-
-=head2 $self->determine_passing()
-
-Calculates whether the test file has passed and caches it in the passing()
-slot.
-
-=cut
-
-sub determine_passing
-{
-    my $self = shift;
-    $self->passing($self->_calc_passing() ? 1 : 0);
-}
-
-=head2 $self->update_skip_reason($detail)
-
-Updates the skip reason according to the detail $detail.
-
-=cut
-
-sub update_skip_reason
-{
-    my $self = shift;
-    my $detail = shift;
-
-    if( $detail->type() eq 'skip' )
-    {
-        my $reason = $detail->reason();
-        if (!defined($self->skip_reason()))
-        {
-            $self->skip_reason($reason);
-        }
-        elsif ($self->skip_reason() ne $reason)
-        {
-            $self->skip_reason('various reasons');
-        }
-    }
-}
-
-=head2 $self->last_detail()
-
-Returns the last detail.
-
-=cut
-
-sub last_detail
-{
-    my $self = shift;
-    return $self->details()->[-1];
-}
-
-sub _is_enormous_event_num
-{
-    my $self = shift;
-
-    return 
-    (
-        ($self->_event->number > 100_000)
-            &&
-        ($self->_event->number > ($self->max()||100_000))
-    );
-}
-
-sub _update_todo_event
-{
-    my ($self) = @_;
-
-    my $event = $self->_event;
-
-    $self->inc_field('todo');
-    if ( $event->is_actual_ok() )
-    {
-        $self->inc_field('bonus');
-    }
-
-    return;
-}
-
-sub _update_skip_event
-{
-    my ($self) = @_;
-
-    $self->inc_field('skip');
-
-    return;
-}
-
-sub _update_by_labeled_test_event
-{
-    my $self = shift;
-
-    my $event = $self->_event;
-
-    if ($event->has_todo())
-    {
-        $self->_update_todo_event();
-    }
-    elsif ($event->has_skip())
-    {
-        $self->_update_skip_event();
-    }
-
-    return;
-}
-
-sub _inc_seen
-{
-    my $self = shift;
-
-    $self->inc_field('seen');
-}
-
-sub _is_event_pass
-{
-    my $self = shift;
-
-    return 
-    (
-        $self->_event->is_ok() ||
-        $self->_is_event_todo() ||
-        $self->_event->has_skip()
-    );
-}
-
-sub _is_event_todo
-{
-    my $self = shift;
-    
-    return $self->_event->has_todo();
-}
-
-sub _update_if_pass
-{
-    my $self = shift;
-
-    if ($self->_is_event_pass())
-    {
-        $self->inc_field('ok');
-    }
-
-    return;
-}
-
-sub _init_details_obj_instance
-{
-    my ($self, $args) = @_;
-    return Test::Run::Straps::StrapsDetailsObj->new($args);
-}
-
-sub _update_details
-{
-    my $self = shift;
-
-    my $event = $self->_event;
-
-    my $details =
-        $self->_init_details_obj_instance(
-            {
-                ok          => $self->_is_event_pass(),
-                actual_ok   => _def_or_blank(scalar($event->is_ok())),
-                name        => _def_or_blank( $event->description ),
-                # $event->directive returns "SKIP" or "TODO" in uppercase
-                # and we expect them to be in lowercase.
-                type        => lc(_def_or_blank( $event->directive )),
-                reason      => _def_or_blank( $event->explanation ),
-            },
-        );
-
-    assert( defined( $details->ok() ) && defined( $details->actual_ok() ) );
-    $self->details()->[$event->number - 1] = $details;
-
-    return;
-}
-
-sub _handle_enormous_event_num
-{
-    my $self = shift;
-
-    return $self->_enormous_num_cb->();
-}
-
-sub _update_details_wrapper
-{
-    my $self = shift;
-
-    if ($self->_is_enormous_event_num())
-    {
-        $self->_handle_enormous_event_num();
-    }
-    else
-    {
-        $self->_update_details();
-    }
-}
-
-sub _handle_event_main
-{
-    my $self = shift;
-
-    $self->_inc_seen();
-    $self->_update_by_labeled_test_event();
-    $self->_update_if_pass();
-    $self->_update_details_wrapper();
-}
-
-=head2 $self->_handle_event({event => $event, enormous_num_cb => sub {...}});
-
-Updates the state of the details using a new TAP::Parser event - $event .
-C<enormous_num_cb> points to a subroutine reference that is the callback for
-handling enormous numbers.
-
-=cut
-
-sub handle_event
-{
-    my ($self, $args) = @_;
-
-    my $event = $args->{event};
-    my $callback = $args->{enormous_num_cb};
-
-    $self->_event($event);
-    $self->_enormous_num_cb($callback);
-
-    $self->_handle_event_main();
-    
-    # Cleanup to avoid circular loops, etc.
-    $self->_event(undef);
-    $self->_enormous_num_cb(undef);
-}
-
-1;
-
-__END__
-
 =head1 SEE ALSO
 
 L<Test::Run::Base::Struct>, L<Test::Run::Obj>, L<Test::Run::Core>
 
 =head1 LICENSE
 
-This file is freely distributable under the same terms as Perl 5 itself.
+This file is licensed under the MIT X11 License:
+
+http://www.opensource.org/licenses/mit-license.php
 
 =head1 AUTHOR
 
@@ -372,3 +460,5 @@ Shlomi Fish, L<http://www.shlomifish.org/>.
 
 =cut
 
+1;
+
